import { Clipboard, Dialog, showDialog } from '@jupyterlab/apputils';
import { NotebookPanel } from '@jupyterlab/notebook';
import { LabIcon } from '@jupyterlab/ui-components';
import { MarkdownCell, CodeCell } from '@jupyterlab/cells';
import { Widget } from '@lumino/widgets';
import React from 'react';
import { showMoreOptions } from './MoreOptions';
import moreSVGstr from '../style/icon/jupyter_moreicon.svg';
import { requestAPI } from './jupyterlab-dbaccess';
const CODE_SNIPPETS_CONTAINER = 'jp-codeDatasetsContainer';
const BUTTON_CLASS = 'jp-codeDatasetsContainer-button';
const ACTION_BUTTONS_WRAPPER_CLASS = 'jp-codeDatasetsContainer-action-buttons';
const ACTION_BUTTON_CLASS = 'jp-codeDatasetsContainer-actionButton';
const CODE_SNIPPET_MORE_OPTIONS = 'jp-codeDataset-options';
const CODE_SNIPPET_MORE_OTPIONS_CONTENT = 'jp-codeDataset-more-options-content';
const CODE_SNIPPET_MORE_OTPIONS_COPY = 'jp-codeDataset-more-options-copy';
const CODE_SNIPPET_MORE_OTPIONS_INSERT = 'jp-codeDataset-more-options-insert';
const CODE_SNIPPET_MORE_OTPIONS_DELETE = 'jp-codeDataset-more-options-delete';
// 图标
const moreOptionsIcon = new LabIcon({
    name: 'custom-ui-components:moreSelectOptions',
    svgstr: moreSVGstr
});
// CodeSnippetDisplay
export class ProdectionTreeDisplay extends React.Component {
    constructor(props) {
        super(props);
        // 在Notebook中插入代码片段
        this.insertCodeSnippet = async (snippet) => {
            const widget = this.props.getCurrentWidget();
            const snippetStr = this.createCodeSnippets(snippet);
            if (widget instanceof NotebookPanel) {
                const notebookWidget = widget;
                const notebookCell = notebookWidget.content.activeCell;
                // editor
                const notebookCellEditor = notebookCell.editor;
                if (notebookCell instanceof CodeCell) {
                    this.verifyLanguageAndInsert(snippet, notebookCellEditor);
                }
                else if (notebookCell instanceof MarkdownCell) {
                    notebookCellEditor.replaceSelection('```' + snippet.language + '\n' + snippetStr + '\n```');
                }
                else {
                    notebookCellEditor.replaceSelection(snippetStr);
                }
            }
            else {
                this.showErrDialog('插入失败，请选择正确的容器界面');
            }
        };
        // 插入
        this.verifyLanguageAndInsert = async (snippet, editor) => {
            const snippetStr = this.createCodeSnippets(snippet);
            editor.replaceSelection(snippetStr);
        };
        // 错误提示
        this.showErrDialog = (errMsg) => {
            return showDialog({
                title: '错误提示',
                body: errMsg,
                focusNodeSelector: 'error_dialog',
                buttons: [Dialog.cancelButton({ label: '关闭' })]
            });
        };
        // 渲染树
        this.renderCodeSnippet = (node, id) => {
            const child = node.children || [];
            return (React.createElement("div", { className: `node_list ${child.length === 0 ? 'child_node' : ''}`, style: child.length === 0 ? { borderBottom: '1px solid #eee' } : { borderBottom: '0px solid #fff' }, key: id },
                this.renderTiggerIcon(child),
                this.renderRadioBtn(node),
                React.createElement("span", { className: "text_item" }, node.text),
                this.renderItemBtn(child, node, id),
                child.map((item) => {
                    return (this.renderCodeSnippet(item, item.id));
                })));
        };
        this.state = {
            searchValue: '',
            filterTags: [],
            snippetList: [],
            isLoading: false,
            isNoData: true
        };
    }
    // 组件挂载
    componentDidMount() {
        this.getProTreeList();
    }
    ;
    // 获取产品树列表
    getProTreeList() {
        this.setState({ isLoading: true });
        requestAPI(`get_tree_list`).then((res) => {
            try {
                const result = JSON.parse(res);
                if (result.code === 200) {
                    this.setState({ isNoData: false });
                    this.setState({ snippetList: result.data || [] });
                }
                else {
                    this.tipMessage('error', '错误提示', result.result);
                }
                this.setState({ isLoading: false });
            }
            catch (_a) {
                this.setState({ isLoading: false });
                this.tipMessage('error', '错误提示', res);
            }
        }).catch(() => {
            this.setState({ isLoading: false });
            this.tipMessage('error', '错误提示', '获取产品数据失败');
        });
    }
    // 统一提示弹框
    tipMessage(type, title, content) {
        let btn = null;
        if (type === 'error')
            btn = Dialog.cancelButton({ label: '关闭', caption: 'cancel' });
        if (type === 'success')
            btn = Dialog.cancelButton({ label: '关闭', caption: 'ok' });
        showDialog({ title: title, body: content, buttons: [btn] });
    }
    //设置开启弹出框的位置
    _setOptionsPosition(event) {
        const target = event.target;
        let top;
        if (target.tagName === 'path') {
            top = target.getBoundingClientRect().top + 10;
        }
        else {
            top = target.getBoundingClientRect().top + 18;
        }
        if (top > 0.7 * window.screen.height) {
            top -= 120;
        }
        const leftAsString = target.getBoundingClientRect().left.toString(10) + 'px';
        const topAsString = top.toString(10) + 'px';
        document.documentElement.style.setProperty('--more-options-top', topAsString);
        document.documentElement.style.setProperty('--more-options-left', leftAsString);
    }
    // 树形节点打开
    setOpen(e) {
        const parent = e.target.parentNode;
        parent.className = parent.className + ' active';
    }
    // 树形节点关闭
    setClose(e) {
        const parent = e.target.parentNode;
        parent.className = 'node_list';
    }
    // 树形节点"展开/关闭"开关按钮渲染
    renderTiggerIcon(child) {
        if (child.length > 0) {
            return [
                React.createElement("span", { className: "open_icon tigger_icon", onClick: (event) => { this.setOpen(event); } }),
                React.createElement("span", { className: "close_icon tigger_icon", onClick: (event) => { this.setClose(event); } })
            ];
        }
    }
    // 树形父级节点radio选择按钮渲染
    renderRadioBtn(node) {
        if (node.children !== undefined) {
            return (
            // <span className={`node_radio_btns ${node.id === this.state.selectedId ? 'node_radio_selected' : ''}`}
            //   onClick={() => { this.setState({ selectedId: node.id }) }}></span>
            React.createElement("span", null));
        }
    }
    // 树形叶子节点右侧按钮组渲染
    renderItemBtn(child, node, id) {
        if (child.length === 0) {
            const buttonClasses = BUTTON_CLASS;
            const actionButtons = [
                {
                    title: '插入, 复制',
                    icon: moreOptionsIcon,
                    onClick: (event) => {
                        showMoreOptions({ body: new OptionsHandler(this, node) });
                        this._setOptionsPosition(event);
                    }
                }
            ];
            if (node.type !== 'product' && node.type !== 'dataTable')
                return null;
            return (React.createElement("div", { className: ACTION_BUTTONS_WRAPPER_CLASS, id: id }, actionButtons.map(btn => {
                return (React.createElement("button", { key: btn.title, title: btn.title, className: buttonClasses + ' ' + ACTION_BUTTON_CLASS, onClick: (event) => {
                        btn.onClick(event);
                    } },
                    React.createElement(btn.icon.react, { tag: "span", elementPosition: "center", width: "16px", height: "16px" })));
            })));
        }
    }
    // 关闭dropdown菜单
    removeOptionsNode() {
        const temp = document.getElementsByClassName(CODE_SNIPPET_MORE_OPTIONS)[0];
        if (!temp.classList.contains('inactive')) {
            temp.classList.add('inactive');
        }
    }
    // 开启dropdown菜单
    createOptionsNode(codeSnippet) {
        const body = document.createElement('div');
        const optionsContainer = document.createElement('div');
        optionsContainer.className = CODE_SNIPPET_MORE_OTPIONS_CONTENT;
        // 插入
        const insertSnip = document.createElement('div');
        insertSnip.className = CODE_SNIPPET_MORE_OTPIONS_INSERT;
        insertSnip.textContent = '插入';
        insertSnip.onclick = () => {
            this.insertCodeSnippet(codeSnippet);
            this.removeOptionsNode();
        };
        // 复制
        const copySnip = document.createElement('div');
        copySnip.className = CODE_SNIPPET_MORE_OTPIONS_COPY;
        copySnip.textContent = '复制';
        copySnip.onclick = () => {
            Clipboard.copyToSystem(this.createCodeSnippets(codeSnippet));
            alert('复制到剪切板');
            this.removeOptionsNode();
        };
        // 删除
        const deleteSnip = document.createElement('div');
        deleteSnip.className = CODE_SNIPPET_MORE_OTPIONS_DELETE;
        deleteSnip.textContent = '删除';
        deleteSnip.onclick = () => {
        };
        // 将下拉框选项按钮添加到列表中
        optionsContainer.appendChild(insertSnip);
        optionsContainer.appendChild(copySnip);
        optionsContainer.appendChild(deleteSnip);
        body.append(optionsContainer);
        return body;
    }
    // 拼接"插入"的代码内容
    createCodeSnippets(node) {
        if (node.type === 'product') {
            const comment = `#读取 ${node.comment}\n`;
            const code = `data_${node.text.split('.')[0]} = explore_platform.read_data_product(app_type_en = '${node.app_type_en}', data_dir_en = '${node.data_dir_en}','${node.text}')`;
            return comment + code;
        }
        if (node.type === 'dataTable') {
            const comment = `#读取 ${node.comment}\n`;
            const code = `table_${node.text} = explore_platform.read_table(app_type_en = '${node.app_type_en}', data_dir_en = '${node.data_dir_en}', db_table_id='${node.text}')`;
            return comment + code;
        }
    }
    // 节点新增
    // private addNode() {
    //   if (!this.state.selectedId) {
    //     return this.showErrDialog('请选择需要新增的算子节点')
    //   }
    // }
    // // 节点删除
    // private delNode() {
    //   if (!this.state.selectedId) {
    //     return this.showErrDialog('请选择需要删除的算子节点')
    //   }
    // }
    // // 节点上传
    // private uploadNode() {
    //   if (!this.state.selectedId) {
    //     return this.showErrDialog('请选择需要上传的算子节点')
    //   }
    // }
    // // 节点下载
    // private downloadNode() {
    //   if (!this.state.selectedId) {
    //     return this.showErrDialog('请选择需要下载的算子节点')
    //   }
    // }
    // 树形模块
    render() {
        return (React.createElement("div", { className: this.state.isLoading ? 'loading_list_container' : 'normal_container' },
            React.createElement("div", { className: "custom_loading_mask" },
                React.createElement("div", { className: "custom_load_effect" },
                    React.createElement("span", null),
                    React.createElement("span", null),
                    React.createElement("span", null),
                    React.createElement("span", null),
                    React.createElement("span", null),
                    React.createElement("span", null),
                    React.createElement("span", null),
                    React.createElement("span", null))),
            React.createElement("div", { className: "dataset_panel_btns" },
                React.createElement("span", { className: "operator_list_btn operator_refresh_btn", title: "\u5237\u65B0", onClick: () => { this.getProTreeList(); } })),
            React.createElement("div", { className: this.state.isNoData ? 'is_hidden_SnippetsContainer' : CODE_SNIPPETS_CONTAINER },
                React.createElement("div", null, this.state.snippetList.map((node) => this.renderCodeSnippet(node, node.id)))),
            React.createElement("div", { className: !this.state.isNoData ? 'is_hidden_SnippetsContainer' : 'empty_container' },
                React.createElement("span", null, "\u6682\u65E0\u6570\u636E"))));
    }
}
class OptionsHandler extends Widget {
    constructor(display, codeSnippet) {
        super({ node: display.createOptionsNode(codeSnippet) });
    }
}
